@@ -122,6 +122,7 @@ from ._api import (
     get_project_anatomy_presets,
     get_default_anatomy_preset_name,
     get_project_anatomy_preset,
+    get_built_in_anatomy_preset,
     get_build_in_anatomy_preset,
     get_project_root_overrides,
     get_project_roots_by_site,
@@ -366,6 +367,7 @@ __all__ = (
     "get_project_anatomy_presets",
     "get_default_anatomy_preset_name",
     "get_project_anatomy_preset",
+    "get_built_in_anatomy_preset",
     "get_build_in_anatomy_preset",
     "get_project_root_overrides",
     "get_project_roots_by_site",
